/*
% Copyright 2009-2010, Jon Sneyers
% 
% This file is part of APOPCALEAPS.
% 
% APOPCALEAPS is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% APOPCALEAPS is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with APOPCALEAPS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <cstdlib>
#include <cstdio>
#include <iostream>
#include <string>
#include <sstream>
#include <fstream>

#include "interface.h"
#include "param_tuning/param_tune_window.cxx"



static void quit() {
    exit(0);
}
static void refresh_query() {
    std::ostringstream qstrs;
    qstrs << "\n";  // just so it displays nicer in the query box
    if (v_drums ->value() == 1) {qstrs << "voice(drums), "
                                       << "shortest_duration(drums," << sn_drums->mvalue()->label() << "), ";
                                }
    if (v_bass  ->value() == 1) {qstrs << "voice(bass), set_range(bass,c,3,-"
                                       << r_bass_low->value() << "," << r_bass_up->value() << "), "
                                       << "instrument(bass,'" << i_bass->mvalue()->label() << "'), "
                                       << "max_jump(bass," << mj_bass->value() << "), "
                                       << "shortest_duration(bass," << sn_bass->mvalue()->label() << "), ";
                                 if (mra_bass->value()==1) qstrs << "max_repeat(bass," << mr_bass->value() << "), ";
                                }
    if (v_chords->value() == 1) {qstrs << "voice(chords), "
                                       << "instrument(chords,'" << i_chords->mvalue()->label() << "'), "
                                       << "chord_style(" << (chords_offbeat->value() == 1 ? "offbeat" : "onbeat") << "), "
                                       << "shortest_duration(chords," << sn_chords->mvalue()->label() << "), ";
                                }
    if (v_melody->value() == 1) {qstrs << "voice(melody), set_range(melody,c,4,-"
                                       << r_melody_low->value() << "," << r_melody_up->value() << "), "
                                       << "instrument(melody,'" << i_melody->mvalue()->label() << "'), "
                                       << "max_jump(melody," << mj_melody->value() << "), "
                                       << "shortest_duration(melody," << sn_melody->mvalue()->label() << "), ";
                                 if (mra_melody->value()==1) qstrs << "max_repeat(melody," << mr_melody->value() << "), ";
                                }

    qstrs << "meter(" << meter1->mvalue()->label() << "," << meter2->mvalue()->label() << "), ";
    //if (play->value() == 1) 
    qstrs << "play, ";
    //if (show->value() == 1) 
    qstrs << "show, ";
    if (major->value() == 1) qstrs << "key(major), ";
    if (minor->value() == 1) qstrs << "key(minor), ";
    qstrs << "repeats(" << repeats->value() <<"), ";
    qstrs << "tempo(" << tempo->value() << "), ";
    qstrs << "chaos(overall," << randomness->value() << "), ";
    qstrs << "chaos(melody," << randomness_melody->value() << "), ";
    qstrs << "chaos(drums," << randomness_drums->value() << "), ";
    qstrs << "chaos(bass," << randomness_bass->value() << "), ";
    qstrs << "chaos(chords," << randomness_chords->value() << "), ";
    qstrs << "measures(" << measures->value() << ")";
    std::string qstr = qstrs.str();


    Fl_Text_Buffer* newtext =  new Fl_Text_Buffer(100);
    newtext->text(qstr.c_str());
    query->buffer(newtext);
}
static void compose() {
    progress->label("  Composing: making query");
    progress->value(1);
//    progress->show();
    Fl::flush();
    refresh_query();
//    save_parameters();
    Fl_Text_Buffer*  q = query->buffer();
    q->savefile("goal",q->length());

    progress->label("  Composing: recompiling program");
    progress->value(2);
    Fl::flush();
    system("./handler1");

    progress->label("  Composing: running program");
    progress->value(4);
    Fl::flush();
    system("./handler2");

    progress->label("  Composing: calling Lilypond");
    progress->value(7);
    Fl::flush();
    system("./handler3");

    if (play->value()==1) {
        progress->label("  Composing: calling TiMidity++");
        progress->value(8);
        Fl::flush();
        system("./handler4");
    }
    if (show->value()==1) {
        progress->label("  Composing: calling GV");
        progress->value(9);
        Fl::flush();
        system("./handler5");
    }
    progress->label("  Composing: done");
    progress->value(10);
    Fl::flush();
    Fl::wait(3);
//    progress->hide();
    progress->label("");
    progress->value(0);

}
void save_query_parameters() {
  std::ofstream file;
  file.open("saved_query.query");
  file.precision(1000);
  file << static_cast<int>(v_drums->value())   << "\n";
  file << sn_drums->value()   << "\n";
  file << static_cast<int>(v_bass->value())   << "\n";
  file << r_bass_low->value()   << "\n";
  file << r_bass_up->value()   << "\n";
  file << i_bass->value()   << "\n";
  file << mj_bass->value()   << "\n";
  file << sn_bass->value()   << "\n";
  file << static_cast<int>(mra_bass->value())   << "\n";
  file << mr_bass->value()   << "\n";
  file << static_cast<int>(v_chords->value())   << "\n";
  file << i_chords->value()   << "\n";
  file << static_cast<int>(chords_offbeat->value())   << "\n";
  file << sn_chords->value()   << "\n";
  file << static_cast<int>(v_melody->value())   << "\n";
  file << r_melody_low->value()   << "\n";
  file << r_melody_up->value()   << "\n";
  file << i_melody->value()   << "\n";
  file << mj_melody->value()   << "\n";
  file << sn_melody->value()   << "\n";
  file << static_cast<int>(mra_melody->value())   << "\n";
  file << mr_melody->value()   << "\n";
  file << meter1->value()   << "\n";
  file << meter2->value()   << "\n";
  file << static_cast<int>(major->value())   << "\n";
  file << static_cast<int>(minor->value())   << "\n";
  file << repeats->value()   << "\n";
  file << randomness->value()   << "\n";
  file << randomness_melody->value()   << "\n";
  file << randomness_bass->value()    << "\n";
  file << randomness_chords->value()   << "\n";
  file << randomness_drums->value()    << "\n";
  file << tempo->value()    << "\n";
  file << measures->value()   << "\n";
  file.close();
}
void load_query_parameters() {
  std::ifstream file;
  file.open("saved_query.query");

  double v;
  file >> v; v_drums->value(v)   ;
  file >> v; sn_drums->value(v)   ;
  file >> v; v_bass->value(v)   ;
  file >> v; r_bass_low->value(v)   ;
  file >> v; r_bass_up->value(v)   ;
  file >> v; i_bass->value(v)   ;
  file >> v; mj_bass->value(v)   ;
  file >> v; sn_bass->value(v)   ;
  file >> v; mra_bass->value(v)   ;
  file >> v; mr_bass->value(v)   ;
  file >> v; v_chords->value(v)   ;
  file >> v; i_chords->value(v)   ;
  file >> v; chords_offbeat->value(v)   ;
  file >> v; sn_chords->value(v)   ;
  file >> v; v_melody->value(v)   ;
  file >> v; r_melody_low->value(v)   ;
  file >> v; r_melody_up->value(v)   ;
  file >> v; i_melody->value(v)   ;
  file >> v; mj_melody->value(v)   ;
  file >> v; sn_melody->value(v)   ;
  file >> v; mra_melody->value(v)   ;
  file >> v; mr_melody->value(v)   ;
  file >> v; meter1->value(v)   ;
  file >> v; meter2->value(v)   ;
  file >> v; major->value(v)   ;
  file >> v; minor->value(v)   ;
  file >> v; repeats->value(v)   ;
  file >> v; randomness->value(v)   ;
  file >> v; randomness_melody->value(v)   ;
  file >> v; randomness_bass->value(v)    ;
  file >> v; randomness_chords->value(v)   ;
  file >> v; randomness_drums->value(v)    ;
  file >> v; tempo->value(v)    ;
  file >> v; measures->value(v)   ;

  file.close();
}        

static void init_saved_songs() {
    saved_song->filter("*.apo");
    saved_song->load("./saved_songs",fl_numericsort);
    saved_song->remove(1);
}
static void init_saved_styles() {
    saved_styles->filter("*.s");
    saved_styles->load("./saved_styles",fl_numericsort);
    saved_styles->remove(1);
}
static void init_saved_queries() {
    saved_queries->filter("*.q");
    saved_queries->load("./saved_queries",fl_numericsort);
    saved_queries->remove(1);
}
static void init_training_set() {
    training_set->filter("*.apo");
    training_set->load("./training_set",fl_numericsort);
    training_set->remove(1);
}

static void save() {
        system("./handler_save");
        init_saved_songs();
}
static void save_style() {
        save_parameters();
        system("./handler_save_style");
        init_saved_styles();
}
static void save_query() {
        save_query_parameters();
        system("./handler_save_query");
        init_saved_queries();
}
static void learn() {
    progress->label("  Learning: creating training data");
    progress->value(1);
//    progress->show();
    Fl::flush();
        system("./handler_learn1");
    progress->label("  Learning: doing EM");
    progress->value(3);
    Fl::flush();
        system("./handler_learn2");
    progress->label("  Learning: loading learned parameters");
    progress->value(9);
    Fl::flush();
         load_parameters_compact();
         save_parameters();
    progress->label("  Learning: done");
    progress->value(10);
    Fl::flush();
    Fl::wait(3);
//    progress->hide();
    progress->label("");
    progress->value(0);
}

static void save2train() {
    std::ostringstream c;
    c << "./handler_save2train "
      <<  saved_song->text(saved_song->value());
    system(c.str().c_str());
    init_training_set();
}
static void train2save() {
    std::ostringstream c;
    c << "./handler_trainrm "
      <<  training_set->text(training_set->value());
    system(c.str().c_str());
    init_training_set();
}

static void play_song() {
    system("./handler4");
}
static void show_song() {
    system("./handler5");
}
static void stop_song() {
    system("./handler_stop");
}
static void open_songs() {
    system("./handler_open_songs");
    init_saved_songs();
}    
static void open_styles() {
    system("./handler_open_styles");
    init_saved_styles();
}    
static void open_queries() {
    system("./handler_open_queries");
    init_saved_queries();
}    
static void goto_website() {
    system("./handler_website");
}

static void loadsaved() {
    std::ostringstream c;
    c << "./handler_load "
      <<  saved_song->text(saved_song->value());
    system(c.str().c_str());
}
static void loadtraining() {
    std::ostringstream c;
    c << "./handler_load_training "
      <<  training_set->text(training_set->value());
    system(c.str().c_str());
}
static void load_style() {
    std::ostringstream c;
    c << "./handler_load_style "
      <<  saved_styles->text(saved_styles->value());
    system(c.str().c_str());
    load_parameters_compact();
    save_parameters();
}

static void load_query() {
    std::ostringstream c;
    c << "./handler_load_query "
      <<  saved_queries->text(saved_queries->value());
    system(c.str().c_str());
    load_query_parameters();
    refresh_query();
}



#include "interface.cxx"
